/*
 * file ProviderFactory.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;

/**
 * A factory class for creating instances of a WVCM provider.
 * 
 * This is a convenience class used to simplify the creation of an instance of a WVCM provider. 
 * A provider is identified by its class name and is runtime loaded. 
 * 
 * @since 1.0
 */
public final class ProviderFactory {

    // no instantiation permitted
    private ProviderFactory() {}

    /**
     * Provides callback functions that can be invoked by a provider.
     */
    public interface Callback {

        /**
         * Authentication information for the current user.
         */
        public interface Authentication {
            /**
             * Returns the login name of the current user.
             * 
             * @return the login name of the current user.
             */
            public String loginName();

            /**
             * Returns the password of the current user.
             * 
             * @return the password of the current user.
             */
            public String password();
        }

        /**
         * Get authentication information for the current user.
         * <p>
         * The provider calls Callback.getAuthentication, and the client returns
         * an object that implements the Authentication interface.
         * The provider will then call the Authentication interface methods
         * on that object to obtain authentication information for the current user.
         * 
         * @param realm an identifier for the authentication realm that is requesting credentials.
         *  A single provider instance can access resources in different authentication realms,
         *  so the user must be provided with the name of the authentication realm in order to
         *  determine what credentials to provide.
         * @param retryCount the number of times the provider has unsuccessfully 
         * attempted to get authentication for this operation.
         * This parameter helps the client decide when to provide a more detailed authentication
         * dialog, or potentially abort the operation rather than making additional attempts
         * to get credentials from the user. 
         * @return user and password authentication interface.
         *  If null is returned, the user wishes to login anonymously.
         * @throws WvcmException if the user aborted the authentication request.
         */
        public Authentication getAuthentication(String realm, int retryCount) throws WvcmException;

    }

    /**
     * Create a WVCM provider with the given class name.
     * 
     * @param providerName the fully-qualified class name of the provider.
     * @param callback the object containing the getAuthentication
     * method that the Provider will call whenever it needs
     * to authenticate the current user.
     * @return an instance of the Provider whose class name is providerName.
     * @throws Exception if no class of the given name can be found, loaded, or initialized.
     */
    public static Provider createProvider(
            String providerName,
            Callback callback)
    throws Exception {
        try {
            return (Provider) Class
            .forName(providerName)
            .getConstructor(Callback.class)
            .newInstance(callback);
        } catch (InvocationTargetException ex) {
            Throwable cause = ex.getCause();
            if (cause instanceof WvcmException)
                throw (WvcmException)cause;
            else
                throw ex;
        }
    }

    /**
     * Get a WVCM provider with the given class name, with specified initializers.
     * 
     * @param providerName the class name of the provider.
     * @param callback the object containing the getAuthentication
     * method that the Provider will call whenever it needs
     * to authenticate the current user.
     * @param initArgs a Map containing provider-specific initialization values.
     * @return an instance of the Provider whose class name is providerName.
     * @throws Exception if no class of the given name can be found, loaded, or initialized.
     * 
     * @since 1.1
     */
    public static Provider createProvider(
            String providerName,
            Callback callback,
            Map<String, String> initArgs
            )
    throws Exception {
        try {
            return (Provider) Class
            .forName(providerName)
            .getConstructor(Callback.class, Map.class)
            .newInstance(callback, initArgs);
        } catch (InvocationTargetException ex) {
            Throwable cause = ex.getCause();
            if (cause instanceof WvcmException)
                throw (WvcmException)cause;
            else
                throw ex;
        }
    }
}
